package com.flexdms.jsfutils;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.view.facelets.AttributeHandler;
import javax.faces.view.facelets.FaceletContext;
import javax.faces.view.facelets.TagAttribute;
import javax.faces.view.facelets.TagConfig;
import javax.faces.view.facelets.TagException;
import javax.faces.view.facelets.TagHandler;

/**
 *  A tag handler like f:attribute, but it evaluates the value expression immediately and 
 *  set the result as attribute value. f:attribute sets the value expression to component 
 *  ValueExpression map. Mainly used inside of c:forEach.
 *  
 * @author jason
 */
public class BuildAttributeHandler extends TagHandler implements
		AttributeHandler
{

	private final TagAttribute name;

	private final TagAttribute value;
	private final TagAttribute ignoreNull;
	private final TagAttribute skipIfExist;

	public BuildAttributeHandler(TagConfig config)
	{
		super(config);
		this.name = this.getRequiredAttribute("name");
		this.value = this.getRequiredAttribute("value");
		this.ignoreNull=this.getAttribute("ignoreNull");
		this.skipIfExist=this.getAttribute("skipIfExist");
	}

	@Override
	public void apply(FaceletContext ctx, UIComponent parent)
			throws IOException
	{
		if (parent == null)
		{
			throw new TagException(this.tag, "Parent UIComponent was null");
		}

		// only process if the parent is new to the tree
		if (parent.getParent() == null)
		{
			String n = getAttributeName(ctx);
			if (!parent.getAttributes().containsKey(n))
			{
				
				if (this.value.isLiteral())
				{
					parent.getAttributes().put(n, this.value.getValue());
				} else
				{
					Object value=this.value.getValueExpression(ctx, Object.class).getValue(FacesContext.getCurrentInstance().getELContext());
					boolean ignoreNull=true;
					if (this.ignoreNull!=null && this.ignoreNull.getValue(ctx)!=null)
					{
						ignoreNull=Utils.stringToBoolean(this.ignoreNull.getValue(ctx), false);
					}
					boolean skipIfExist=true;
					if (this.skipIfExist!=null && this.skipIfExist.getValue(ctx)!=null)
					{
						skipIfExist=Utils.stringToBoolean(this.skipIfExist.getValue(ctx), false);
					}
					if (value==null && ignoreNull)
					{
						return;
					}
					if (parent.getAttributes().containsKey(n) && skipIfExist)
					{
						return;
					}
					parent.getAttributes().put(n, value);
				}
			}
		}

	}

	@Override
	public String getAttributeName(FaceletContext ctx)
	{
		return this.name.getValue(ctx);
	}

}
