package com.flexdms.jsfutils;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.faces.FactoryFinder;
import javax.faces.application.Application;
import javax.faces.application.Resource;
import javax.faces.application.ResourceHandler;
import javax.faces.component.UIComponent;
import javax.faces.component.UIForm;
import javax.faces.component.UIPanel;
import javax.faces.context.FacesContext;
import javax.faces.view.facelets.Facelet;
import javax.faces.view.facelets.FaceletFactory;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;


public class FaceletUtils
{

	private static String[] dissectResource(String resource)
	{
		int i=resource.indexOf('/');
		String libandname[]=new String[2];
		libandname[0]=resource.substring(0, i);
		libandname[1]=resource.substring(i+1);
		return libandname;
	}
	/**
	 * create a component from resource
	 * @param resource such as "library/name.xhtml"
	 * @return an component created from resource
	 */
	public static UIComponent resouceToComponent(String resource)
	{
		return faceletToComponent(resouceToFacet(resource));
	}


	private static Facelet resouceToFacet(String resource)
	{
		FacesContext fctx = FacesContext.getCurrentInstance();
		Application app = fctx.getApplication();
		FaceletFactory faceletFactory = (FaceletFactory) FactoryFinder
				.getFactory(FactoryFinder.FACELET_FACTORY);
		ResourceHandler rh=app.getResourceHandler();
		String libAndName[]=dissectResource(resource);
		Resource res = rh.createResource(libAndName[1], libAndName[0]);
		Facelet f;
		try
		{
			f = faceletFactory.getFacelet(res.getURL());
		} catch (IOException e)
		{
			throw new RuntimeException(e);
		}
		return f;

	}

	
	private static UIComponent faceletToComponent(Facelet f)
	{
		FacesContext fctx = FacesContext.getCurrentInstance();
		Application app = fctx.getApplication();
		UIPanel panel = (UIPanel) app.createComponent(UIPanel.COMPONENT_TYPE);
		try
		{
			f.apply(fctx, panel);
		} catch (IOException e)
		{
			throw new RuntimeException(e);
		}
		UIComponent newComponent = panel.getChildren().remove(0);

		return newComponent;
	}

	/**
	 * 
	 * return a string which navigates from fromfile to tofile.
	 * 
	 * @param fromfile
	 *            absolute file path
	 * @param tofile
	 *            absolute file path
	 * 
	 * @return
	 */
	private static String fileToFile(String fromfile, String tofile)
	{
		if (tofile.startsWith(File.separator))
		{
			tofile = tofile.substring(1);
		}
		if (fromfile.startsWith(File.separator))
		{
			fromfile = fromfile.substring(1);
		}
		String[] froms = fromfile.split(File.separator);
		String[] tos = tofile.split(File.separator);
		List<String> fs = new LinkedList<String>();
		int i = 0;
		for (i = 0; i < froms.length - 1; i++)
		{

			// strip common directory.
			if (!froms[i].equals(tos[i]))
			{
				break;
			}
		}

		if (i == froms.length - 1)
		{
			// reach the last file, all path are the same.
			fs.add(".");
		} else
		{
			for (int j = i; j < froms.length - 1; j++)
			{
				fs.add("..");
			}
		}
		for (int j = i; j < tos.length; j++)
		{
			fs.add(tos[j]);
		}

		return StringUtils.join(fs, "/");
	}

	/**
	 * return a composite component with attributes
	 * You can not pass Object as attribute value. If you need pass Object, 
	 * use ValueExpression
	 * 
	 * @param library library name,
	 * @param name resource name
	 * @param attrs could be null, only string value is accepted.
	 * @return
	 */
	public static UIComponent constructCompositeObject(String library,
			String name, Map<String, Object> attrs)
	{

		UIComponent result = null;
		FaceletFactory faceletFactory = (FaceletFactory) FactoryFinder
				.getFactory(FactoryFinder.FACELET_FACTORY);

		if (attrs == null)
		{
			attrs = Collections.emptyMap();
		}
		result = faceletFactory.createComponent(
				"http://java.sun.com/jsf/composite/" + library, name, attrs);
		return result;

	}

	/**
	 * 
	 * Construct a composite comonent given attribute and context-relative facet
	 * file.
	 * 
	 * @param library
	 *            library of the composite component
	 * @param name
	 *            name of the composite component
	 * @param attrs
	 *            a series of attrs value for the component.
	 * @param facets
	 *            a series of paired value. The first value in pair is facet
	 *            name. The second value is CONTEXT-RELATIVE facet file.
	 * @return UIComponent represent the constructed composite component
	 * @throws IOException
	 */
	public static UIComponent applyFacetFileToCompositeComponent(
			String library, String name, Map<String, Object> attrs,
			Object... facets)
	{
		UIComponent result = null;

		FaceletFactory faceletFactory = (FaceletFactory) FactoryFinder
				.getFactory(FactoryFinder.FACELET_FACTORY);
		FacesContext context = FacesContext.getCurrentInstance();
		Application app = context.getApplication();

		String tempId = context.getViewRoot().createUniqueId(context, name);
		File tempFile = createTempFile(context);
		String content = getCompositeComponentString(library, name, attrs,
				context, tempFile, tempId, facets);

		try
		{
			FileUtils.write(tempFile, content);
			URL furl = tempFile.toURI().toURL();
			Facelet f = faceletFactory.getFacelet(furl);
			UIForm form = (UIForm) app.createComponent(UIForm.COMPONENT_TYPE);
			form.setPrependId(false);
			form.setId(Utils.createUniqueId());
			f.apply(context, form);
			result = form.findComponent(tempId);
			form.getChildren().clear();

			return result;
		} catch (IOException e)
		{
			throw new RuntimeException(e);
		} finally
		{
			tempFile.delete();
		}
	}

	/**
	 * 
	 * onstruct a composite comonent given attribute and facet objects
	 * 
	 * @param library
	 *            library of the composite component
	 * @param name
	 *            name of the composite component
	 * @param attrs
	 *            a series of attrs value for the component.
	 * @param facets
	 *            a series of paired value. The first value in pair is facet
	 *            name. The second value is UIComponent OBJECT
	 * @return
	 * @throws IOException
	 */
	public static UIComponent applyFacetObjectToCompositeComponent(
			String library, String name, Map<String, Object> attrs,
			Map<String, UIComponent> facets)
	{

		UIComponent result = null;
		FaceletFactory faceletFactory = (FaceletFactory) FactoryFinder
				.getFactory(FactoryFinder.FACELET_FACTORY);

		if (attrs == null)
		{
			attrs = Collections.emptyMap();
		}
		result = faceletFactory.createComponent(
				"http://java.sun.com/jsf/composite/" + library, name, attrs);

		if (facets!=null && !facets.isEmpty())
		{
			result.getFacets().putAll(facets);
		}

		return result;

	}

	private static String getCompositeComponentString(String library,
			String name, Map<String, Object> attrs, FacesContext context,
			File tempFile, String tempId, Object... facets)
	{
		StringBuilder sb = new StringBuilder();
		sb.append("<?xml version='1.0' encoding='UTF-8' ?>");
		sb.append("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">");
		sb.append("<html xmlns=\"http://www.w3.org/1999/xhtml\"\n");
		sb.append(" xmlns:f=\"http://java.sun.com/jsf/core\"\n");
		sb.append(" xmlns:ui=\"http://java.sun.com/jsf/facelets\"\n");
		sb.append("      xmlns:comp=\"http://java.sun.com/jsf/composite/")
				.append(library).append("\">\n");
		sb.append("  <comp:").append(name).append(" ");

		// have all the attributes
		if (attrs != null && !attrs.isEmpty())
		{
			for (Map.Entry<String, Object> attr : attrs.entrySet())
			{
				sb.append(attr.getKey()).append("=\"")
						.append(attr.getValue().toString()).append("\"");
			}
		}
		// give component An ID

		sb.append(" id=\"").append(tempId).append("\" >\n");

		// add the facet
		for (int i = 0; i < facets.length; i++)
		{

			sb.append("<f:facet name=\"").append(facets[i++].toString())
					.append("\" >\n");
			sb.append("<ui:include src=\"")
					.append(fileToFile(
							tempFile.getAbsolutePath(),
							context.getExternalContext().getRealPath(
									facets[i].toString()))).append("\" />\n");
			sb.append("</f:facet>\n");
		}

		// end the tag.
		sb.append("</comp:").append(name).append(">");
		sb.append("</html>");

		return sb.toString();
	}

	private static File createTempFile(FacesContext context)
	{
		File tmpDir = (File) context.getExternalContext().getApplicationMap()
				.get("javax.servlet.context.tempdir");
		File tempFile;
		try
		{
			tempFile = File.createTempFile("faceletutils", ".xhtml", tmpDir);
		} catch (IOException e)
		{
			throw new RuntimeException(e);
		}
		return tempFile;
	}

	/**
	 * 
	 * @param template
	 *            context relative template
	 * @param definitions
	 *            a series of pair value<name: definition file>
	 * @return a component which containing the tempate rendering component
	 * @throws IOException
	 */
	public static UIComponent instantiateTemplate(String template,
			Object... definitions)
	{
		FacesContext ctx = FacesContext.getCurrentInstance();
		FaceletFactory faceletFactory = (FaceletFactory) FactoryFinder
				.getFactory(FactoryFinder.FACELET_FACTORY);

		UIPanel panel = (UIPanel) ctx.getApplication().createComponent(
				UIPanel.COMPONENT_TYPE);
		File tempFile = createTempFile(ctx);
		String content = getTemplateComponentString(template, ctx, tempFile,
				definitions);
		try
		{
			FileUtils.write(tempFile, content);
			URL furl = tempFile.toURI().toURL();
			Facelet f = faceletFactory.getFacelet(furl);
			f.apply(ctx, panel);
			return panel;
		} catch (IOException e)
		{
			throw new RuntimeException(e);
		} finally
		{
			tempFile.delete();
		}
	}

	private static String getTemplateComponentString(String template,
			FacesContext context, File tempFile, Object... definitions)
	{
		StringBuilder sb = new StringBuilder();
		sb.append("<?xml version='1.0' encoding='UTF-8' ?>");
		sb.append("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">");
		sb.append("<html xmlns=\"http://www.w3.org/1999/xhtml\"\n");
		sb.append(" xmlns:f=\"http://java.sun.com/jsf/core\"\n");
		sb.append(" xmlns:ui=\"http://java.sun.com/jsf/facelets\">\n");
		sb.append("  <ui:composition template=\"")
				.append(fileToFile(tempFile.getAbsolutePath(), context
						.getExternalContext().getRealPath(template)))
				.append("\">");

		// add the facet
		for (int i = 0; i < definitions.length; i++)
		{
			sb.append("<ui:define name=\"").append(definitions[i++].toString())
					.append("\" >\n");
			sb.append("<ui:include src=\"")
					.append(fileToFile(
							tempFile.getAbsolutePath(),
							context.getExternalContext().getRealPath(
									definitions[i].toString())))
					.append("\" />\n");
			sb.append("</ui:define>\n");
		}

		// end the tag.
		sb.append("</ui:composition>");
		sb.append("</html>");

		return sb.toString();
	}
}
