package com.flexdms.jsfutils;

import java.beans.FeatureDescriptor;
import java.util.Iterator;

import javax.el.ELContext;
import javax.el.ELResolver;
import javax.el.PropertyNotWritableException;
import javax.faces.component.UIComponent;
import javax.faces.component.NamingContainer;
import javax.faces.context.FacesContext;

/*
 * Usage: 
 * 	#{_nc} resolve to NamingContainer from current component
 *  #{A._nc} resolve to NamingContainer from A's Parent.
 *  
 *  #{_nc._nc}; this is the second form. The first _nc is resolved to A, the Second _nc 
 *  is resolved to A's parent nc. This form is very useful for datatable.
 *  
 *  For example, suppose a component inside  datatable try to refer a component A (id=a) at the same level as datatable
 *  We could do it like this
 *  ":#{_nc._nc.clientId}:a
 *  
 *  The first _nc is refer the datatable, the second _nc refer upper NamingContainer.
 *  :#{_nc._nc.clientId} get the absolute client ID. Then we append component A ID
 *  
 *  TODO: hanlde root case.
 *  
 */
public class NameContainerELResolver extends ELResolver
{

	
	public static final String NamingContainer="_nc";
	
	
	public boolean shouldHandle(Object base, Object property)
	{
		if (base==null && property!=null && property instanceof String)
		{
			if (property.equals(NamingContainer))
			{
				return true;
			}
		} 
		
			
		if (base!=null && base instanceof UIComponent && property!=null && property.equals(NamingContainer))
		{
			return true;
		}
		return false;
	}
	
	public NamingContainer resolveBase(Object base, Object property)
	{
		UIComponent comp=UIComponent.getCurrentComponent(FacesContext.getCurrentInstance());
		while(comp!=null)
		{
			if (comp instanceof NamingContainer)
			{
				return (NamingContainer)comp;
			}
			comp=comp.getParent();
		}
		return null;
	}
	
	public NamingContainer resolveParent(Object base, Object property)
	{
		UIComponent comp=(UIComponent)base;
		comp=comp.getParent();
		while(comp!=null)
		{
			if (comp instanceof NamingContainer)
			{
				return (NamingContainer)comp;
			}
			comp=comp.getParent();
		}
		return null;
	}
	
	@Override
	public Object getValue(ELContext context, Object base, Object property)
	{
		if (shouldHandle(base, property))
		{
			context.setPropertyResolved(true);
			if (base==null)
			{
				return resolveBase(base, property);
			} else
			{
				return resolveParent(base, property);
			}
		}
		return null;
	}

	@Override
	public Class<?> getType(ELContext context, Object base, Object property)
	{
		if (shouldHandle(base, property))
		{
			/*
			 * No object is accepted.
			 */
			context.setPropertyResolved(true);
			return NamingContainer.class;
		}
		return null;
	}

	@Override
	public void setValue(ELContext context, Object base, Object property,
			Object value)
	{

		if (shouldHandle(base, property))
		{
			throw new PropertyNotWritableException();
		}

	}

	@Override
	public boolean isReadOnly(ELContext context, Object base, Object property)
	{
		return true;
	}

	@Override
	public Iterator<FeatureDescriptor> getFeatureDescriptors(ELContext context,
			Object base)
	{
		return null;
	}

	@Override
	public Class<?> getCommonPropertyType(ELContext context, Object base)
	{
		if (base == null)
		{
			return String.class;
		}
		return null;
	}
}
