package com.flexdms.jsfutils;

import java.util.HashMap;
import java.util.Map;

import javax.el.MethodExpression;
import javax.el.ValueExpression;
import javax.faces.component.UIComponent;
import javax.faces.component.UIOutput;
import javax.faces.component.UIViewRoot;
import javax.faces.component.html.HtmlPanelGroup;
import javax.faces.context.FacesContext;

public class Utils
{
	/**
	 * @return a HtmlPanelGroup with layout set to block
	 */
	public static HtmlPanelGroup createPanel()
	{
		HtmlPanelGroup panel = new HtmlPanelGroup();
		panel.setLayout("block");
		return panel;
	}
	/**
	 * Add StyleClass to component without replace existing style class
	 * @param comp: UIComponent
	 * @param sc: StyleClass
	 */
	public static void addClass(UIComponent comp, String sc)
	{
		Object styleClass = comp.getAttributes().get("styleClass");
		if (styleClass != null)
		{
			comp.getAttributes().put("styleClass",
					styleClass.toString() + " " + sc);
		} else
		{
			comp.getAttributes().put("styleClass", sc);
		}
	}

	/**
	 * 
	 * @param component
	 * @param sc
	 * @return whether component has the StyleClass or not
	 */
	public static boolean hasClass(UIComponent component, String sc)
	{
		Object styleClass = component.getAttributes().get("styleClass");
		if (styleClass==null)
		{
			return false;
		}
		String path[]=styleClass.toString().split("\\s+");
		for(String p:path)
		{
			if (p.equals(sc))
				return true;
		}
		return false;
	}
	/**
	 * 
	 * @param exp expression string
	 * @return value Expression
	 */
	public static ValueExpression createValueExpression(String exp)
	{
		FacesContext fc = FacesContext.getCurrentInstance();
		ValueExpression ve = fc.getApplication().getExpressionFactory()
				.createValueExpression(fc.getELContext(), exp, Object.class);
		return ve;
	}
	/**
	 * 
	 * @param exp method expression string
	 * @return an Method expression 
	 */
	public static MethodExpression createActionExpression(String exp)
	{
		FacesContext fc = FacesContext.getCurrentInstance();
		MethodExpression ve = fc.getApplication().getExpressionFactory().createMethodExpression(fc.getELContext(), exp, String.class, new Class[0]);
				
		return ve;
	}

	
	public static UIComponent createScriptComponent(String library,
			String name, String target)
	{
		FacesContext ftc = FacesContext.getCurrentInstance();

		UIComponent component = ftc.getApplication().createComponent(ftc,
				UIOutput.COMPONENT_TYPE, "javax.faces.resource.Script");
		component.getAttributes().put("library", library);
		component.getAttributes().put("name", name);
		component.getAttributes().put("target", target);

		return component;
	}

	public static UIComponent createStyleComponent(String library, String name, String target)
	{
		FacesContext ftc = FacesContext.getCurrentInstance();

		UIComponent component = ftc.getApplication().createComponent(ftc,
				UIOutput.COMPONENT_TYPE, "javax.faces.resource.Stylesheet");
		component.getAttributes().put("library", library);
		component.getAttributes().put("name", name);
		component.getAttributes().put("target", target);

		return component;
	}
	
	/**
	 * 
	 * @param exp method expression string
	 * @return an Method expression which can be used for ajax call or normal action listener.
	 */
	public static MethodExpressionListener createListener(String methodExp)
	{
		return new MethodExpressionListener(methodExp);
	}
	
	/**
	 * 
	 * @return whether current request is ajax call. JSF provides this but you need to create PartialViewContext.
	 */
	public static boolean isAjaxRequest()
	{
		
		return "partial/ajax".equals(FacesContext.getCurrentInstance().getExternalContext().getRequestHeaderMap().get("Faces-Request"));
		
	}
	/**
	 * 
	 * @return whether current request is partial call. JSF provides this but you need to create PartialViewContext.
	 */
	public static boolean isPartialRequest()
	{
		if (isAjaxRequest())
		{
			return true;
		}
		return "partial/process".equals(FacesContext.getCurrentInstance().getExternalContext().getRequestHeaderMap().get("Faces-Request"));
		
	}

	public static String IDPREFIX="fims";
	/**
	 *http://java.net/jira/browse/JAVASERVERFACES-2611 
	 * @return a view-scoped view id.
	 */
	public static String createUniqueId()
	{
		UIViewRoot root=FacesContext.getCurrentInstance().getViewRoot();
		Map<String, Object> viewMap=root.getViewMap(true);
		if (viewMap.containsKey(IDPREFIX))
		{
			int i=(Integer)viewMap.get(IDPREFIX);
			viewMap.put(IDPREFIX, ++i);
			return IDPREFIX+"_"+i;
		} else
		{
			int i=1;
			viewMap.put(IDPREFIX, i);
			return IDPREFIX+"_"+i;
		}
	}
	
	/**
	 * 
	 * @param component starting component or null
	 * @param fc FacesContext
	 * @return first component from component stack which is in view tree.
	 */
	public static UIComponent getInViewComponent(UIComponent component, FacesContext fc)
	{
		if (component!=null && component.isInView())
		{
			return component;
		}
		return getInViewComponent(fc);
	}
	
	private static UIComponent getInViewComponent(FacesContext fc)
	{
		UIComponent component=UIComponent.getCurrentComponent(fc);
		if (component==null)
		{
			//How can this be null
			//When dyanmically add UIData to the system. 
			//UIData.invokeOnComponent will setRowIndex(-1) withput push current component to stack.
			return null;
		}
		if (component.isInView())
		{
			return component;
		}
		component.popComponentFromEL(fc);
		UIComponent inViewComp=null;
		try
		{
			inViewComp=getInViewComponent(fc);
		} finally
		{
			//push back to maintain the component stack.
			component.pushComponentToEL(fc, component);
		}
		return inViewComp;
	}
	
	public static boolean stringToBoolean(String v, boolean strict)
	{
		if (v==null)
			return false;
		v=v.toUpperCase();
		if (v.equals("YES")||v.equals("Y")||v.equals("1")
				||v.equals("TRUE")||v.equals("T")||v.equals("ON")
				||v.equals("CHECKED")|| v.equals("CHECK")
				||v.equals("SELECTED")||v.equals("SELECT"))
			return true;
		
		if (v.equals("NO")||v.equals("N")||v.equals("0")
				||v.equals("FALSE")||v.equals("F")||v.equals("OFF")
				||v.equals("UNCHECKED")|| v.equals("UNCHECK")
				||v.equals("UNSELECTED")||v.equals("UNSELECT"))
			return false;
		if (!strict)
			return false;
		throw new RuntimeException("String is not a boolean value");
	}
	
}
